package tokenservicelogic

import (
	"app-cloudep-permission-server/internal/domain"
	"context"

	ers "code.30cm.net/digimon/library-go/errors"

	"app-cloudep-permission-server/gen_result/pb/permission"
	"app-cloudep-permission-server/internal/svc"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetUserTokensByDeviceIdLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewGetUserTokensByDeviceIdLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetUserTokensByDeviceIdLogic {
	return &GetUserTokensByDeviceIdLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

type getUserTokensByDeviceIdReq struct {
	DeviceID string `json:"device_id" validate:"required"`
}

// GetUserTokensByDeviceId 取得目前所對應的 DeviceID 所存在的 Tokens
func (l *GetUserTokensByDeviceIdLogic) GetUserTokensByDeviceId(in *permission.DoTokenByDeviceIDReq) (*permission.Tokens, error) {
	if err := l.svcCtx.Validate.ValidateAll(&getUserTokensByDeviceIdReq{
		DeviceID: in.GetDeviceId(),
	}); err != nil {
		return nil, ers.InvalidFormat(err.Error())
	}

	uidTokens, err := l.svcCtx.TokenRedisRepo.GetAccessTokensByDeviceID(l.ctx, in.GetDeviceId())
	if err != nil {
		return nil, err
	}

	tokens := make([]*permission.TokenResp, 0, len(uidTokens))
	for _, v := range uidTokens {
		tokens = append(tokens, &permission.TokenResp{
			AccessToken:  v.AccessToken,
			TokenType:    domain.TokenTypeBearer,
			ExpiresIn:    int32(v.ExpiresIn),
			RefreshToken: v.RefreshToken,
		})
	}

	return &permission.Tokens{
		Token: tokens,
	}, nil
}
